"""Top-level package for Cfg4Py."""
from cfg4py.core import (
    RemoteConfigFetcher, enable_logging, config_remote_fetcher, init,
    update_config, RedisConfigFetcher, config_server, envar, _cfg_obj)

__author__ = """Aaron Yang"""
__email__ = 'code@jieyu.ai'
__version__ = "__version__ = '0.6.0'"


def get_instance():
    return _cfg_obj


__all__ = ['RemoteConfigFetcher', 'enable_logging', 'config_remote_fetcher', 'init', 'update_config',
           'RedisConfigFetcher', 'config_server', 'envar', 'get_instance']



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    "watchdog>=0.9.0",
    "apscheduler>=3.6.3",
    "fire>=0.2.1",
    "ruamel.yaml==0.16.0"
]

setup_requirements = []

test_requirements = []

setup(
    author="Aaron Yang",
    author_email='aaron_yang@jieyu.ai',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="A python config module support hierarchical configuration and multi-environment deployment",
    install_requires=requirements,
    license="BSD license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='cfg4py',
    name='cfg4py',
    packages=find_packages(include=['cfg4py', 'cfg4py.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/jieyu_tech/cfg4py',
    version='0.6.0',
    zip_safe=False,
    entry_points={
        'console_scripts': ['cfg4py=cfg4py.command_line:main']
    }
)



import logging
import os
import sys

import fire
from ruamel.yaml import YAML
from typing import Optional

from cfg4py import init, enable_logging, envar

enable_logging()


class Command:

    def __init__(self):
        self.resource_path = os.path.normpath(os.path.join(os.path.dirname(__file__), 'resources/'))
        self.yaml = YAML(typ='safe')  # default, if not specfied, is 'rt' (round-trip)
        self.yaml.default_flow_style = False

        with open(os.path.join(self.resource_path, 'template.yaml'), 'r') as f:
            self.templates = self.yaml.load(f)

        self.transformed = self._transform()

    def build(self, config_dir: str):
        """
        Compile configuration files into a python class file, which is used by IDE's auto-complete function

        Args:
            config_dir: The folder where your configuration files located

        Returns:

        """
        if not os.path.exists(config_dir):
            print(f"path {config_dir} not exists")
            sys.exit(-1)

        count = 0
        for f in os.listdir(config_dir):
            if f.startswith("default") or f.startswith("dev") or f.startswith("test") or f.startswith("production"):
                print(f"found {f}")
                count += 1

        if count > 0:
            print(f"{count} files found in total")
        else:
            print("the folder contains no valid configuration files")
            sys.exit(-1)

        try:
            # set server role temporarily
            os.environ["__cfg4py_server_role__"] = "DEV"

            init(config_dir)
            sys.path.insert(0, config_dir)
            # noinspection PyUnresolvedReferences
            from cfg4py_auto_gen import Config
            print(f"Config file is built with success and saved at {os.path.join(config_dir, 'cfg4py_auto_gen')}")
        except Exception as e: # pragma: no cover
            logging.exception(e)
            print("Config file built failure.")

    def _choose_dest_dir(self, dst):
        if dst is None:
            dst = input("Where should I save configuration files?\n")

        if os.path.exists(dst):
            for f in os.listdir(dst):
                if f in ['defaults.yaml', 'dev.yaml', 'test.yaml', 'production.yaml']:
                    print(f"The folder provided already contains {f}, please choose clean one.")
                    return None
            return dst
        else:
            create = input("Path not exists, create('Q' to exit)? [Y/n]")
            if create.upper() == 'Y':
                os.makedirs(dst, exist_ok=True)
                return dst
            elif create.upper() == 'Q':
                sys.exit(-1)
            else:
                return None

    def scaffold(self, dst: Optional[str]):
        """
        Creates initial configuration files based on our choices.
        Args:
            dst:

        Returns:

        """
        print("Creating a configuration boilerplate:")
        dst = self._choose_dest_dir(dst)
        while dst is None:
            dst = self._choose_dest_dir(dst)

        yaml = YAML(typ='safe')  # default, if not specfied, is 'rt' (round-trip)
        with open(os.path.join(self.resource_path, 'template.yaml'), 'r') as f:
            templates = yaml.load(f)

        print("Which flavors do you want?")
        print("-" * 20)
        prompt = """
        0  - console + rotating file logging
        10 - redis/redis-py (gh://andymccurdy/redis-py)
        11 - redis/aioredis (gh://aio-libs/aioredis)
        20 - mysql/PyMySQL (gh://PyMySQL/PyMySQL)
        30 - postgres/asyncpg (gh://MagicStack/asyncpg)
        31 - postgres/psycopg2 (gh://psycopg/psycopg2)
        40 - mq/pika (gh://pika/pika)
        50 - mongodb/pymongo (gh://mongodb/mongo-python-driver)
        """
        print(prompt)
        chooses = input("Please choose flavors by index, separated each by a comma(,):\n")
        flavors = {}
        mapping = {
            "0": "logging",
            "1": "redis",
            "2": "mysql",
            "3": "postgres",
            "4": "mq",
            "5": "mongodb"
        }
        for index in chooses.strip(' ').split(','):
            if index == "0":
                flavors["logging"] = templates["logging"]
                continue

            try:
                major = mapping[index[0]]
                minor = int(index[1])
                flavors[major] = list(templates[major][minor].values())[0]
            except (ValueError, KeyError):
                print(f"Wrong index {index}, skipped.")
                continue

        with open(os.path.join(dst, "defaults.yaml"), 'w') as f:
            f.writelines("#auto generated by Cfg4Py: https://github.com/jieyu-tech/cfg4py\n")
            yaml.dump(flavors, f)

        print(f"Cfg4Py has generated the following files under {dst}:")
        print("defaults.yaml")
        for name in ['dev.yaml', 'test.yaml', 'production.yaml']:
            with open(os.path.join(dst, name), 'w') as f:
                f.writelines("#auto generated by Cfg4Py: https://github.com/jieyu-tech/cfg4py\n")
                print(name)

        with open(os.path.join(dst, "defaults.yaml"), 'r') as f:
            print("Content in defaults.yaml")
            for line in f.readlines():
                print(line.replace('\n', ''))

    def _show_supported_config(self):
        print("Support the following configurations:")
        for key in self.templates.keys():
            item = self.templates.get(key)
            if isinstance(item, dict):
                print(f"  {key}")
            elif isinstance(item, list):
                sub_keys = []
                for sub_item in item:
                    sub_keys.append(f"{key}/{list(sub_item.keys())[0]}")
                print(f"  {key}: {', '.join(sub_keys)}")

    def _transform(self):
        transformed = {}
        for key in self.templates:
            if isinstance(self.templates[key], dict):
                transformed[key] = self.templates[key]
            elif isinstance(self.templates[key], list):
                for item in self.templates[key]:
                    item_key = list(item.keys())[0]
                    transformed[f"{key}/{item_key}"] = item

        return transformed

    def hint(self, what: str = None, usage: bool = False):
        """show a cheat sheet for configurations.
        for example:
            cfg4py hint mysql
        this will print how to configure PyMySQL
        :param what
        :param usage
        """

        if what is None or ((what not in self.templates) and what not in self.transformed):
            return self._show_supported_config()

        usage_key = f"{what}_usage"

        if usage_key in self.templates and usage:
            print("Usage:", self.templates.get(usage_key))

        if what in self.templates:
            self.yaml.dump(self.templates[what], sys.stdout)

        if usage_key in self.transformed and usage:
            print("Usage:", self.transformed.get(usage_key))

        if what in self.transformed:
            self.yaml.dump(self.transformed[what], sys.stdout)

    def set_server_role(self):
        print("please add the following line into your .bashrc:\n")
        print(f"export {envar}=DEV\n")
        print("You need to change DEV to TEST | PRODUCTION according to its actual role accordingly")

def main():
    cmd = Command() # pragma: no cover
    fire.Fire({ # pragma: no cover
        "build":           cmd.build,
        "scaffold":        cmd.scaffold,
        "hint":            cmd.hint,
        "set_server_role": cmd.set_server_role
    })


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""Tests for `cfg4py` package."""
import shutil
import unittest
import os
import tempfile

import cfg4py
from cfg4py import core
import logging
from unittest.mock import patch

from cfg4py.command_line import Command

logger = logging.getLogger(__name__)


def early_jump(msg):
    logger.info("Mock exit: %s", msg)
    raise SystemExit


class TestCfg4Py(unittest.TestCase):
    """Tests for `cfg4py` package."""

    def setUp(self):
        """Set up test fixtures, if any."""
        cfg4py.enable_logging()
        os.environ['__cfg4py_server_role__'] = 'TEST'
        self.resource_path = os.path.join(os.path.dirname(__file__), "../cfg4py/resources/")
        self.resource_path = os.path.normpath(self.resource_path)
        self.home = os.path.normpath(os.path.join(os.path.dirname(__file__), '..'))
        self.output = os.path.join(tempfile.gettempdir(), 'cfg4py')
        shutil.rmtree(self.output, ignore_errors=True)

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_create_config(self):
        """Test something."""
        logger.info("resource_path is %s", self.resource_path)
        cfg = cfg4py.init(self.resource_path)
        self.assertEqual(cfg.services.redis.host, '127.0.0.1')

    def test_001_update_config(self):
        conf = {
            "aaron":    {
                "surname": "yang"
            },
            "services": {
                "redis": {
                    "host": "localhost"
                }
            }
        }

        cfg = cfg4py.update_config(conf)
        self.assertEqual('localhost', cfg.services.redis.host)

    def test_001_build(self):
        conf = {
            "aaron":    {
                "surname": "yang"
            },
            "services": {
                "redis": {
                    "host": "localhost"
                }
            }
        }

        cfg4py.update_config(conf)
        core.build(os.path.join(self.resource_path, "cfg4py_auto_gen.py"))
        try:
            import sys
            sys.path.insert(0, self.resource_path)
            # noinspection PyUnresolvedReferences
            from cfg4py_auto_gen import Config
            # no exception means the file has been generated successfully
            self.assertTrue(True)
        except Exception:
            self.assertTrue(False)

    def test_002_create_config(self):
        from cfg4py.resources.cfg4py_auto_gen import Config
        os.environ['__cfg4py_server_role__'] = 'DEV'
        cfg: Config = cfg4py.init(self.resource_path)
        print("cfg.services.redis.host is", cfg.services.redis.host)

    def test_003_config_remote_fetcher(self):
        from cfg4py import RedisConfigFetcher
        from redis import StrictRedis
        import json
        r = StrictRedis('localhost')
        r.set("my_app_config", json.dumps({
            "services": {
                "redis":  {
                    "host": "192.168.3.1",
                },
                "redis2": {
                    "host": "192.168.3.2"
                }
            }
        }))

        cfg = cfg4py.init(self.resource_path)
        fetcher = RedisConfigFetcher(key="my_app_config")
        logger.info("configuring a remote fetcher")
        cfg4py.config_remote_fetcher(fetcher, 1)
        import time
        time.sleep(2)
        logger.info("please check log output to see if update_config is called at least 1 once")
        # the one from dev.yaml
        self.assertEqual(cfg.services.redis.host, '127.0.0.1')
        # the one from redis
        self.assertEqual(cfg.services.redis2.host, '192.168.3.2')

    def test_004_command_hint(self):
        cmd = Command()
        logger.info("please check the output")
        cmd.hint('pip')
        cmd.hint('pip', True)
        cmd.hint('postgres/psycopg2')
        cmd.hint('postgres/psycopg2', True)
        cmd.hint('pip/tshinghua')

    def test_005_sigleton(self):
        cfg1 = cfg4py.get_instance()
        cfg2 = cfg4py.get_instance()

        self.assertEqual(id(cfg1), id(cfg2))

    def test_006_scaffold(self):
        cmd = Command()

        # folder not exists, create
        answers = [self.output, 'Y', '11']
        with unittest.mock.patch('builtins.input', side_effect=answers):
            cmd.scaffold(None)

        self.assertTrue(os.path.exists(os.path.join(self.output, 'defaults.yaml')))

        # files exists, ask for dst again, then we give non-exist path and 'Q' to quit
        answers = ['', 'Q']

        with unittest.mock.patch('builtins.input', side_effect=answers):
            try:
                with unittest.mock.patch('sys.exit', lambda *args: early_jump('user pressed Q')):
                    cmd.scaffold(self.output)
                self.assertTrue(False, 'sys exit not triggered')
            except SystemExit:
                self.assertTrue(True)

        # path exists and the folder is empty
        shutil.rmtree(self.output, ignore_errors=True)
        os.makedirs(self.output, exist_ok=True)
        answers = ['31']
        with unittest.mock.patch('builtins.input', side_effect=answers):
            cmd.scaffold(self.output)

        self.assertTrue(os.path.exists(os.path.join(self.output, 'defaults.yaml')))

        # non exist index
        shutil.rmtree(self.output, ignore_errors=True)
        os.makedirs(self.output, exist_ok=True)
        with unittest.mock.patch('builtins.input', side_effect=['99']):
            cmd.scaffold(self.output)

        # chose logging
        shutil.rmtree(self.output, ignore_errors=True)
        os.makedirs(self.output, exist_ok=True)
        with unittest.mock.patch('builtins.input', side_effect=['0']):
            cmd.scaffold(self.output)

    def test_007_cmd_build(self):
        cmd = Command()

        os.makedirs(self.output)
        with unittest.mock.patch('builtins.input', side_effect=['31']):
            cmd.scaffold(self.output)

        # normal run
        cmd.build(self.output)
        cfg = cfg4py.init(self.output)
        self.assertEqual(cfg.postgres.dsn, "dbname=test user=postgres password=secret")

        # path not exists run
        shutil.rmtree(self.output)
        try:
            with unittest.mock.patch('sys.exit', lambda *args: early_jump('Path not exists')):
                cmd.build(self.output)
            self.assertTrue(False, 'sys.exit not triggered')
        except SystemExit:
            self.assertTrue(True)

        # no files in self.output
        os.makedirs(self.output, exist_ok=True)
        try:
            with unittest.mock.patch('sys.exit', lambda *args: early_jump('no files in folder')):
                cmd.build(self.output)
            self.assertTrue(False, 'sys.exit not triggered')
        except SystemExit:
            self.assertTrue(True)

    def test_008_enable_logging(self):
        os.makedirs(self.output, exist_ok=True)
        cfg4py.enable_logging(log_file=os.path.join(self.output, "cfg4py.log"), file_size=1e-6, file_count=3)
        logger.info("%s", ["test log"])

    def test_009_set_server_role(self):
        cmd = Command()
        cmd.set_server_role()
